import { world, Player, Vector } from "@minecraft/server";

/** @type {{ [k: string]: { damageBase: number; damageMin: number; fallOffMin: number; fallOffMax: number } }} */
const bulletDamage = {
  // citori_725 bullets
  bullet1: {
    damageBase: 22,
    damageMin: 8,
    fallOffMin: 8,
    fallOffMax: 20,
  },
  // p90 bullets
  bullet2: {
    damageBase: 10,
    damageMin: 7,
    fallOffMin: 48,
    fallOffMax: 96,
  },
  // ak bullets
  bullet3: {
    damageBase: 18,
    damageMin: 13,
    fallOffMin: 56,
    fallOffMax: 128,
  },
  // p250 bullets
  bullet4: {
    damageBase: 10,
    damageMin: 6,
    fallOffMin: 32,
    fallOffMax: 56,
  },
  // m4 bullets
  bullet5: {
    damageBase: 16,
    damageMin: 11,
    fallOffMin: 56,
    fallOffMax: 128,
  },
  // mini-gun bullets
  bullet6: {
    damageBase: 20,
    damageMin: 13,
    fallOffMin: 32,
    fallOffMax: 128,
  },
  // deagle bullets
  bullet7: {
    damageBase: 22,
    damageMin: 10,
    fallOffMin: 32,
    fallOffMax: 88,
  },
};

world.afterEvents.projectileHit.subscribe((ev) => {
  const src = ev.source;
  if (!(src instanceof Player)) return;

  const proj = ev.projectile;
  try {
    if (proj.typeId != "ra:bullet.projectile") return;
  } catch {
    return;
  }

  const entityHit = ev.getEntityHit();
  if (!entityHit) return;

  const entity = entityHit.entity;
  try {
    entity.typeId;
  } catch {
    return;
  }
  /** @type {number} */
  const variant = proj.getComponent("minecraft:mark_variant")?.value ?? 0;
  if (variant == 0) return;
  const damageDef = bulletDamage[`bullet${variant}`];
  if (!damageDef) {
    console.warn(`Could not find a bullet damage definition for variant ${variant}`);
    return;
  }

  const dist = Vector.subtract(src.location, ev.location).length();

  let dmg = damageDef.damageBase;
  if (dist > damageDef.fallOffMin) {
    // {x > a: round(max(c+(d-c)*(1-(x-a)/(b-a)),c)),d}
    // const { damageMin: dmin, damageMax: dmax, fallOffMin: fmin, fallOffMax: fmax } = damageDef;
    // dmg = Math.round(Math.max(dmin, dmin + (dmax - dmin) * (1 - (dist - fmin) / (fmax - fmin))));
    const fallOff = Math.max(1 - (dist - damageDef.fallOffMin) / (damageDef.fallOffMax - damageDef.fallOffMin), 0);
    dmg = Math.round(Math.max(dmg * fallOff, damageDef.damageMin));
  }
  const vel = entity.getVelocity();
  entity.applyDamage(dmg, { damagingProjectile: proj, damagingEntity: src });
  entity.clearVelocity();
  entity.applyImpulse(vel);
  // world.sendMessage(`Damage: ${dmg}`);
});
