import { world, MolangVariableMap, Vector, Player } from "@minecraft/server";
import { getMainhand } from "./utils";
import { dirMap } from "./globals";

const ignoreBlocks = [
  "lantern",
  "glass",
  "glass_pane",
  "stained_glass",
  "stained_glass_pane",
  "bamboo",
  "cocoa",
  "carved_pumpkin",
  "chorus_flower",
  "chorus_plant",
  "lit_pumpkin",
  "melon_block",
  "pumpkin",
  "turtle_egg",
  "waterlily",
  "cave_vines",
  "cave_vines_body_with_berries",
  "cave_vines_head_with_berries",
  "small_dripleaf_block",
  "big_dripleaf",
  "chain",
];

/** @type {{ [k:string]: string }} */
const bulletParticle = {
  default: "ra:bullet_hole",
  "ra:p250": "ra:bullet_hole.small",
  "ra:remington_870": "ra:bullet_hole.small",
  "ra:citori_725": "ra:bullet_hole.small",
  "ra:mini_gun": "ra:bullet_hole.large",
};

///////
//
///////

world.afterEvents.projectileHit.subscribe((ev) => {
  const proj = ev.projectile;
  try {
    if (proj.typeId != "ra:bullet.projectile") return;
  } catch {
    return;
  }

  const blockHit = ev.getBlockHit();
  if (!blockHit) return;

  const block = blockHit.block;

  const id = block.typeId.replace("minecraft:", "");
  if (ignoreBlocks.includes(id)) return;

  const { dimension: dim, source } = ev;

  const vars = new MolangVariableMap();
  const face = blockHit.face;
  const { x, y, z } = dirMap[face];
  vars.setVector3("variable.block_face", new Vector(x, y, z));

  if (!(source instanceof Player)) return;

  const gunId = getMainhand(source)?.typeId ?? "minecraft:air";
  const particle = bulletParticle[gunId] ?? bulletParticle.default;

  dim.spawnParticle(particle, ev.location, vars);
});
