import { Dimension, Player, ScoreboardIdentity, ScoreboardObjective, world } from "@minecraft/server";

/**
 * @param {Player} plr
 */
export function getMainhand(plr) {
  const inv = plr.getComponent("inventory");
  if (!inv) return;
  const cont = inv.container;
  return cont.getSlot(plr.selectedSlot);
}

/**
 * @param {string} id
 * @returns
 */
export function getObjective(id) {
  try {
    const obj = world.scoreboard.getObjective(id);
    if (obj) return obj;
  } catch {}
  return world.scoreboard.addObjective(id, id);
}

/**
 *
 * @param {ScoreboardObjective} obj
 * @param {ScoreboardIdentity | undefined} ident
 * @returns
 */
export function getScore(obj, ident) {
  if (!ident) return;
  try {
    return obj.getScore(ident);
  } catch {
    return;
  }
}

/**
 *
 * @param {Dimension} dim
 * @param {string} sound
 * @param {import("@minecraft/server").PlayerSoundOptions} soundOpts
 * @param {import("@minecraft/server").EntityQueryOptions} query
 * @param {import("@minecraft/server").Vector3} location
 */
export function playSound(dim, sound, soundOpts = {}, query = {}, location = { x: 0, y: 0, z: 0 }) {
  for (const p of dim.getPlayers(
    Object.assign(
      {
        location,
        minDistance: 0,
      },
      query
    )
  )) {
    p.playSound(sound, Object.assign({ location }, soundOpts));
  }
}

/**
 *
 * @param {import("@minecraft/server").Vector3} vector
 * @param {number} angle
 * @returns
 */
export function rotateVector(vector, angle) {
  angle = (angle * Math.PI) / 180;
  return {
    x: vector.x * Math.cos(angle) + vector.z * Math.sin(angle),
    y: vector.y,
    z: -vector.x * Math.sin(angle) + vector.z * Math.cos(angle),
  };
}
